#------------------------------------------------------------------------------
# /etc/rc.d/helper-dhcp - common functions to generate dhcp-config
#
# Creation:     2008-10-10 lanspezi
# Last Update:  $Id: helper-dhcp.sh 28630 2013-11-12 22:16:52Z kristov $
#------------------------------------------------------------------------------

# lease-file - check for writeable directory
lease_file ()
{
    case "x$DHCP_LEASES_DIR" in
        x|xauto) DHCP_LEASES_DIR=/var/lib/persistent/dhcp ;;
    esac

    if [ -d "$DHCP_LEASES_DIR" ]
    then
        sh -c "> ${DHCP_LEASES_DIR}/dhcp_testfile" 2> /dev/null
        if [ ! -f "${DHCP_LEASES_DIR}/dhcp_testfile" ]
        then
            log_error "ERROR: cannot write to DHCP_LEASES_DIR"
            return 1
        fi
        rm "${DHCP_LEASES_DIR}/dhcp_testfile"
    else
        if ! mkdir -p "$DHCP_LEASES_DIR"; then
            log_error "ERROR: could not create directory for the leases-file"
            return 1
        fi
    fi
    return 0
}

check_lease_file ()
{
    if [ ! -f "${DHCP_LEASES_DIR}/$1.leases" ]
    then
        if [ "$DHCP_LEASES_VOLATILE" != yes ]; then
            log_error "WARNING: dhcp-leases was not found!"
            log_error "if this is your first boot with OPT_DHCP enabled you can"
            log_error "ignore this message. otherwise your dhcp-leases-file is not"
            log_error "saved permanently. in this case you should read the docu again!"
        fi
        > ${DHCP_LEASES_DIR}/$1.leases
    fi
}

# set all parameters for an wins-server
dhcpd_wins ()
{
    if [ "$OPT_SAMBA" = "yes" -a "$OPT_NMBD" = "yes" ]
    then
        if [ "$NMBD_WINSSERVER" = "yes" -o -n "$NMBD_EXTWINSIP" ]
        then
            if [ "$NMBD_WINSSERVER" = "yes" ]
            then
                wins_server="ROUTER"
            fi
            if [ -n "$NMBD_EXTWINSIP" ]
            then
                wins_server=$NMBD_EXTWINSIP
            fi
        fi
    else
        if [ "$DHCP_WINSSERVER_1" != "" ]
        then
            wins_server=$DHCP_WINSSERVER_1
            if [ "$DHCP_WINSSERVER_2" != "" ]
            then
                wins_server="$wins_server, $DHCP_WINSSERVER_2"
            fi
        fi
    fi
}
