#------------------------------------------------------------------------------
# /etc/rc.d/helper-dns_dnsmasq.sh - configure dnsmasq dns
#
# Creation:     2007-02-03  LanSpezi
# Last Update:  $Id: helper-dns_dnsmasq.sh 42106 2015-10-22 08:44:55Z florian $
#------------------------------------------------------------------------------

create_resolv_dnsserver ()
{
    # create /etc/resolv.dnsmasq
    # forwarder(s) for dnsmasq - changed by ip-up.dnsmasq if USEPEERDNS
    {
       echo "# forwarders for dnsmasq"
       for j in `echo $DNS_FORWARDERS | sed -e 's/\[\(.*\)\]:[0-9]*/\1/g' -e 's/\([0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\):[0-9]*/\1/g'`
       do
           echo "nameserver $j"
       done
    } > /etc/resolv.dnsmasq
    chown dns:dns /etc/resolv.dnsmasq
    chmod 600 /etc/resolv.dnsmasq
}

create_resolv_dnsserver_n ()
{
    # create /etc/dnsmasq.d/dns_resolver.conf
    # forwarder(s) with optinal port for dnsmasq
    # can be changed by ip-up.dnsmasq if USEPEERDNS
    {
        echo "# forwarders for dnsmasq"
        for j in $DNS_FORWARDERS
        do
            echo "server=$j" | sed -e 's/\[\(.*\)\]:\([0-9]*\)/\1#\2/g' -e 's/\([0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\):\([0-9]*\)/\1#\2/g'
        done
    } > /etc/dnsmasq.d/dns_resolver.conf
}

create_dns_forbidden ()
{
    #
    # create /etc/dnsmasq.d/dns_forbidden.conf
    #
    for idx in `seq 1 $DNS_FORBIDDEN_N`
    do
        eval dom='$DNS_FORBIDDEN_'$idx
        echo "local=/$dom/" >> /etc/dnsmasq.d/dns_forbidden.conf
    done
}

create_dns_redirect ()
{
    #
    # create /etc/dnsmasq.d/dns_redirect.conf
    #
    for idx in `seq 1 $DNS_REDIRECT_N`
    do
        eval dom='$DNS_REDIRECT_'$idx
        eval domip='$DNS_REDIRECT_'$idx'_IP'
        echo "address=/$dom/$domip" >> /etc/dnsmasq.d/dns_redirect.conf
    done
}

create_dns_config ()
{
    # create /etc/dnsmasq.conf
    {
        echo "user=dns"
        echo "group=dns"
        if [ -n "`echo $DNS_FORWARDERS | grep -e '\.[0-9][0-9]*:' -e '\]:'`" ]
        then
            create_resolv_dnsserver_n
            echo "no-resolv"
        else
            create_resolv_dnsserver
            echo "resolv-file=/etc/resolv.dnsmasq"
        fi
        echo "no-poll"
        echo "no-negcache"
        echo "log-facility=local0"

#        if [ "$DNS_BOGUS_PRIV" = "yes" ]
#        then
#            echo "bogus-priv"
#        fi
        if [ "$DNS_VERBOSE" = "yes" ]
        then
            echo "log-queries"
        fi

        # Listen-IP's (Interfaces)
        if [ 0"$DNS_LISTEN_N" -ne 0 ]
        then
            if [ "$DNS_BIND_INTERFACES" = "yes" ]
            then
                echo "bind-interfaces"
            fi
            echo "listen-address=127.0.0.1"
            for idx in `seq 1 $DNS_LISTEN_N`
            do
                eval ipaddr='$DNS_LISTEN_'$idx
                case $ipaddr in
                IP_NET_*|*.*.*.*)
                    prot=4
                    translate_ip_net $ipaddr DNS_LISTEN_$idx
                    ;;
                IPV6_NET_*|*:*)
                    prot=6
                    translate_ip6_net $ipaddr DNS_LISTEN_$idx
                    ;;
                esac
                if [ $? -eq 0 ]
                then
                    if ip -$prot addr show | grep -q "$res/"
                    then
                        echo "listen-address=$res"
                    else
                        log_error "DNS_LISTEN_$idx='$ipaddr' ($res) is not a local IP"
                    fi
                fi
            done
        fi

        # Authoritative IP's (Interfaces)
        if [ "$DNS_AUTHORITATIVE" = yes ]
        then
            if translate_ip_net $DNS_AUTHORITATIVE_IPADDR DNS_AUTHORITATIVE_IPADDR
            then
                if ip addr show | grep -q "$res/"
                then
                    echo "auth-server=$DNS_AUTHORITATIVE_NS,$res"
                else
                    log_error "DNS_AUTHORITATIVE_IPADDR='$DNS_AUTHORITATIVE_IPADDR' ($res) is not a local IP"
                fi
            fi
        fi

        # Zone configuration
        if [ 0"$DNS_ZONE_NETWORK_N" -ne 0 ]
        then
            zone_networks=
            sep=
            for idx in `seq 1 $DNS_ZONE_NETWORK_N`
            do
                eval zone_network=\$DNS_ZONE_NETWORK_$idx
                zone_networks="$zone_networks$sep$zone_network"
                sep=,
            done
            echo "auth-zone=$DOMAIN_NAME,$zone_networks"

            if [ "$DNS_MX_SERVER" != "" ]
            then
                # according to the man page, mx-host is necessary for
                # authoritative mode (mx-target alone is not sufficient)
                echo "mx-host=$DOMAIN_NAME"
            fi
        fi

        echo "domain-suffix=$DOMAIN_NAME"
        echo "expand-hosts"

        # Reject (and log) addresses from upstream nameservers which are in the private IP ranges.
        #This blocks an attack where a browser behind a firewall is used to probe machines on the local network.
        echo "stop-dns-rebind"
        # Allow loopback addresses to be returned by external DNS servers
        # (cf. ticket #222)
        echo "rebind-localhost-ok"

        if [ -f /etc/hosts.d/hosts.dns ]
        then
            echo "addn-hosts=/etc/hosts.d/hosts.dns"
        fi
        if [ -f /etc/hosts.d/hosts.extra ]
        then
            echo "addn-hosts=/etc/hosts.d/hosts.extra"
        fi

        if [ -f /etc/hosts.global ]
        then
            mv /etc/hosts.global /etc/hosts.d/hosts.global
            echo "addn-hosts=/etc/hosts.d/hosts.global"
        fi

        # file for hosts added dynamically by the webinterface
        echo "# file for hosts added dynamically by the webinterface" > /etc/hosts.d/hosts.dhcp
        echo "#" >> /etc/hosts.d/hosts.dhcp
        echo "addn-hosts=/etc/hosts.d/hosts.dhcp"

        chown -h dns:dns /etc/hosts.d/*
        chmod 666 /etc/hosts.d/* 2>/dev/null

        if [ "$DNS_MX_SERVER" != "" ]
        then
            echo "localmx"
            echo "mx-target=$DNS_MX_SERVER"
        fi

        # forbidden Domains
        create_dns_forbidden

        # redirect Domains
        create_dns_redirect

        if [ "$DNS_FILTERWIN2K" = "yes" ]
        then
            echo "filterwin2k"
        fi

        # set TTL for local hosts. local hosts the ones from /etc/hosts and every DHCP host
        : ${DNS_LOCAL_HOST_CACHE_TTL:=0}
        echo "local-ttl=$DNS_LOCAL_HOST_CACHE_TTL"

        if [ "$DNS_FORWARD_LOCAL" = "no" ]
        then
            echo "domain-needed"
            # do not forward querys for local-Domain
            echo "local=/$DOMAIN_NAME/"
        fi

        echo "min-port=4096"

        mkdir -p /etc/dnsmasq.d

        # include config-files with special-DNS-configs
        echo "conf-dir=/etc/dnsmasq.d"

    } > /etc/dnsmasq.conf
    chown dns:dns /etc/dnsmasq.conf
    chmod 600 /etc/dnsmasq.conf

    # new version of dns_delegate.conf
    {
        cat << EOF
#----------------------------------------------------------------------------
# /etc/dnsmasq.d/dns_delegate.conf
#----------------------------------------------------------------------------
EOF
        [ 0$DNS_ZONE_DELEGATION_N -eq 0 ] || for i in `seq 1 $DNS_ZONE_DELEGATION_N`
        do
            eval upstream_domain_count='$DNS_ZONE_DELEGATION_'$i'_DOMAIN_N'
            [ 0$upstream_domain_count -eq 0 ] || for x in `seq 1 $upstream_domain_count`
            do
                eval upstream_domain='$DNS_ZONE_DELEGATION_'$i'_DOMAIN_'$x
                eval upstream_server_count='$DNS_ZONE_DELEGATION_'$i'_UPSTREAM_SERVER_N'
                [ 0$upstream_server_count -eq 0 ] || for j in `seq 1 $upstream_server_count`
                do
                    eval upstream_ipport='$DNS_ZONE_DELEGATION_'$i'_UPSTREAM_SERVER_'$j'_IP'
                    # split ip/net and port
                    set `echo $upstream_ipport | sed 's/:/ /'`
                    addr=$1
                    port=$2
                    if translate_ip_net $addr 'DNS_ZONE_DELEGATION_'$i'_UPSTREAM_SERVER_'$j'_IP'
                    then
                        upstream_ipport="$res"
                        if [ -n "$port" ]
                        then
                            upstream_ipport="$upstream_ipport#$port"
                        fi
                        eval upstream_query_ipport='$DNS_ZONE_DELEGATION_'$i'_UPSTREAM_SERVER_'$j'_QUERYSOURCEIP'
                        if [ "x$upstream_query_ipport" != "x" ]
                        then
                            # split ip/net and port
                            case $upstream_query_ipport in
                            \[*\]:*)
                                addr=${upstream_query_ipport%:*}
                                addr=${addr#\[}
                                addr=${addr%\]}
                                port=${upstream_query_ipport##*:}
                                ;;
                            *:*)
                                addr=${upstream_query_ipport%:*}
                                port=${upstream_query_ipport##*:}
                                ;;
                            *)
                                addr=$upstream_query_ipport
                                port=
                                ;;
                            esac

                            if translate_ip_net $addr 'DNS_ZONE_DELEGATION_'$i'_UPSTREAM_SERVER_'$j'_QUERYSOURCEIP'
                            then
                                upstream_query_ipport="$res"
                                if [ -n "$port" ]
                                then
                                    upstream_query_ipport="$upstream_query_ipport#$port"
                                fi
                                echo "server=/$upstream_domain/$upstream_ipport@$upstream_query_ipport"
                            else
                                log_error "$upstream_query_ipport from DNS_ZONE_DELEGATION_$i_UPSTREAM_SERVER_$j_QUERYSOURCEIP will be ignored!"
                            fi
                        else
                            echo "server=/$upstream_domain/$upstream_ipport"
                        fi
                    else
                        log_error "$upstream_ipport from DNS_ZONE_DELEGATION_$i_UPSTREAM_SERVER_$j_IP will be ignored!"
                    fi
                done
                echo "rebind-domain-ok=$upstream_domain"
            done
            eval upstream_network_count='$DNS_ZONE_DELEGATION_'$i'_NETWORK_N'
            [ 0$upstream_network_count -eq 0 ] || for x in `seq 1 $upstream_network_count`
            do
                eval upstream_network='$DNS_ZONE_DELEGATION_'$i'_NETWORK_'$x
                if check_rdns $upstream_network
                then
                    [ 0$upstream_server_count -eq 0 ] || for j in `seq 1 $upstream_server_count`
                    do
                        eval upstream_ipport='$DNS_ZONE_DELEGATION_'$i'_UPSTREAM_SERVER_'$j'_IP'
                        # split ip/net and port
                        set `echo $upstream_ipport | sed 's/:/ /'`
                        addr=$1
                        port=$2
                        if translate_ip_net $addr 'DNS_ZONE_DELEGATION_'$i'_UPSTREAM_SERVER_'$j'_IP'
                        then
                            upstream_ipport="$res"
                            if [ -n "$port" ]
                            then
                                upstream_ipport="$upstream_ipport#$port"
                            fi
                            eval upstream_query_ipport='$DNS_ZONE_DELEGATION_'$i'_UPSTREAM_SERVER_'$j'_QUERYSOURCEIP'
                            if [ "x$upstream_query_ipport" != "x" ]
                            then
                                # split ip/net and port
                                set `echo $upstream_query_ipport | sed 's/:/ /'`
                                addr=$1
                                port=$2
                                if translate_ip_net $addr 'DNS_ZONE_DELEGATION_'$i'_UPSTREAM_SERVER_'$j'_QUERYSOURCEIP'
                                then
                                    upstream_query_ipport="$res"
                                    if [ -n "$port" ]
                                    then
                                        upstream_query_ipport="$upstream_query_ipport#$port"
                                    fi
                                    echo "rev-server=$upstream_network,$upstream_ipport@$upstream_query_ipport"
                                else
                                    log_error "$upstream_query_ipport from DNS_ZONE_DELEGATION_$i_UPSTREAM_SERVER_$j_QUERYSOURCEIP will be ignored!"
                                fi
                            else
                                echo "rev-server=$upstream_network,$upstream_ipport"
                            fi
                        else
                            log_error "$upstream_ipport from DNS_ZONE_DELEGATION_$i_UPSTREAM_SERVER_$j_IP will be ignored!"
                        fi
                    done
                else
                    log_error "$upstream_network from DNS_ZONE_DELEGATION_$i_NETWORK_N will be ignored!"
                fi
            done
        done

        [ 0$DNS_REBINDOK_N -eq 0 ] || for i in `seq 1 $DNS_REBINDOK_N`
        do
            eval sp_domain='$DNS_REBINDOK_'$i'_DOMAIN'
            if [ "x$sp_domain" != "x" ]
            then
                echo "rebind-domain-ok=$sp_domain"
            fi
        done

    } > /etc/dnsmasq.d/dns_delegate.conf

    # do not forward reverse-request for local loopback-adresses
    {
        echo ""
        echo "local=/.127.in-addr.arpa/"
        echo ""
    } >> /etc/dnsmasq.d/dns_delegate.conf

    if [ "$DNS_BOGUS_PRIV" = "yes" ]
     then
         {
             cat << EOF

local=/.10.in-addr.arpa/
local=/.254.169.in-addr.arpa/
local=/.0.0.192.in-addr.arpa/
local=/.2.0.192.in-addr.arpa/
local=/.100.51.198.in-addr.arpa/
local=/.113.0.203.in-addr.arpa/
local=/.16.172.in-addr.arpa/
local=/.17.172.in-addr.arpa/
local=/.18.172.in-addr.arpa/
local=/.19.172.in-addr.arpa/
local=/.20.172.in-addr.arpa/
local=/.21.172.in-addr.arpa/
local=/.22.172.in-addr.arpa/
local=/.23.172.in-addr.arpa/
local=/.24.172.in-addr.arpa/
local=/.25.172.in-addr.arpa/
local=/.26.172.in-addr.arpa/
local=/.27.172.in-addr.arpa/
local=/.28.172.in-addr.arpa/
local=/.29.172.in-addr.arpa/
local=/.30.172.in-addr.arpa/
local=/.31.172.in-addr.arpa/
local=/.168.192.in-addr.arpa/
EOF
         } >> /etc/dnsmasq.d/dns_delegate.conf
     fi

    # create ipset parameters for dnsmasq
    cat /var/run/ipset.list | while read hostexpr ipsetname
    do
        echo "ipset=/$hostexpr/${ipsetname}4,${ipsetname}6" >> /etc/dnsmasq.d/dns_ipset.conf
    done
}

check_rdns ()
{
    case x$1 in
        x) ;;
        x-REMOVE)
            if [ -f /tmp/dnsmasq_rdns.check ]
            then
                rm /tmp/dnsmasq_rdns.check
            fi
            ;;
        x*)
            eval checknet=`netcalc network $1`

            if [ -f /tmp/dnsmasq_rdns.check ] && grep -q $checknet /tmp/dnsmasq_rdns.check
            then
                log_error "You use $checknet more than once for reverse dns delegation - please check your config!"
                return 1
            else
                echo "$checknet" >> /tmp/dnsmasq_rdns.check
                return 0
            fi
            ;;
    esac
}

# Setups the IPv4 firewall chain controlling incoming DNS requests.
# Input:
#   $1 = default policy (ACCEPT or DROP)
dns_setup_chain()
{
    add_chain in-dns-udp
    add_chain in-dns-tcp
    get_count INPUT
    ins_rule filter INPUT "prot:tcp 53 in-dns-tcp" $res "filters DNS TCP requests"
    get_count INPUT
    ins_rule filter INPUT "prot:udp 53 in-dns-udp" $res "filters DNS UDP requests"
    ins_rule filter in-dns-udp $1
    ins_rule filter in-dns-tcp $1
}

# Setups the IPv6 firewall chain controlling incoming DNS requests.
# Input:
#   $1 = default policy (ACCEPT or DROP)
dns_setup_chain6()
{
    add_chain6 in-dns-udp
    add_chain6 in-dns-tcp
    get_count6 INPUT
    ins_rule6 filter INPUT "prot:tcp 53 in-dns-tcp" $res "filters DNS TCP requests"
    get_count6 INPUT
    ins_rule6 filter INPUT "prot:udp 53 in-dns-udp" $res "filters DNS UDP requests"
    ins_rule6 filter in-dns-udp $1
    ins_rule6 filter in-dns-tcp $1
}

conf_firewall_dns4()
{
    if [ 0$DNS_LISTEN_N -eq 0 ]
    then
        dns_setup_chain ACCEPT
    else
        dns_setup_chain DROP

        [ 0$IP_NET_N -eq 0 ] || for idx in `seq 1 $IP_NET_N`
        do
            eval net='$IP_NET_'$idx
            case $net in
            dhcp)
                continue
                ;;
            esac

            net_dev='IP_NET_'$idx'_DEV'
            netip_val='IP_NET_'$idx'_IPADDR'
            if translate_ip_net $netip_val
            then
                net_ip=$res
                for idn in `seq $DNS_LISTEN_N`
                do
                    eval dnslisten='$DNS_LISTEN_'$idn
                    case $dnslisten in
                    IP_NET_*|*.*.*.*)
                        translate_ip_net $dnslisten DNS_LISTEN_${idn} dnslisten_ip
                        if [ "$net_ip" = "$dnslisten_ip" ]
                        then
                            ins_rule filter in-dns-udp "if:$net_dev:any any $dnslisten_ip ACCEPT" 1 "accept UDP DNS requests on $dnslisten_ip"
                            ins_rule filter in-dns-tcp "if:$net_dev:any any $dnslisten_ip ACCEPT" 1 "accept TCP DNS requests on $dnslisten_ip"
                            break
                        fi
                        ;;
                    esac
                done
             fi
        done
    fi
}

conf_firewall_dns6()
{
    if [ 0$DNS_LISTEN_N -eq 0 ]
    then
        dns_setup_chain6 ACCEPT
    else
        dns_setup_chain6 DROP

        [ 0$IPV6_NET_N -eq 0 ] || for idx in `seq 1 $IPV6_NET_N`
        do
            net_dev='IPV6_NET_'$idx'_DEV'
            netip_val='IPV6_NET_'$idx'_IPADDR'
            if translate_ip6_net $netip_val
            then
                net_ip=$res
                for idn in `seq $DNS_LISTEN_N`
                do
                    eval dnslisten='$DNS_LISTEN_'$idn
                    case $dnslisten in
                    IPV6_NET_*|*:*)
                        translate_ip6_net $dnslisten DNS_LISTEN_${idn} dnslisten_ip
                        if [ "$net_ip" = "$dnslisten_ip" ]
                        then
                            ins_rule6 filter in-dns-udp "if:$net_dev:any any $dnslisten_ip ACCEPT" 1 "accept UDP DNS requests on $dnslisten_ip"
                            ins_rule6 filter in-dns-tcp "if:$net_dev:any any $dnslisten_ip ACCEPT" 1 "accept TCP DNS requests on $dnslisten_ip"
                            break
                        fi
                        ;;
                    esac
                done
             fi
        done
    fi
}

conf_firewall_dns ()
{
    [ "$PF_INPUT_ACCEPT_DEF" = "yes" ] && conf_firewall_dns4
    if [ "$OPT_IPV6" = yes ]
    then
        [ "$PF6_INPUT_ACCEPT_DEF" = "yes" ] && conf_firewall_dns6
    fi
}
