#------------------------------------------------------------------------------
# /etc/rc.d/helper-hosts.sh - configure host-names
#
# Creation:     2007-02-03  LanSpezi
# Last Update:  $Id: helper-hosts.sh 33619 2014-10-04 06:33:17Z kristov $
#------------------------------------------------------------------------------

create_dns_hosts ()
{
    if [ "$OPT_DNS" = "yes" ]
    then
        hostfile="/etc/hosts.d/hosts.dns"
    else
        hostfile="/etc/hosts"
    fi

    if [ 0$HOST_N -ne 0 ]
    then

        log_info "writing HOSTS to $hostfile ..."

        mkdir -p /etc/dnsmasq.d

        {
            cat <<EOF
#----------------------------------------------------------------------------
# $hostfile
# automatically created by rc001.hosts
#----------------------------------------------------------------------------
EOF
            [ 0$HOST_N -eq 0 ] || for i in `seq 1 $HOST_N`
            do
                eval hostname='$HOST_'$i'_NAME'

                eval host_domain='$HOST_'$i'_DOMAIN'
                if [ "x$host_domain" != x ]
                then
                    domain="$host_domain"
                else
                    domain="$DOMAIN_NAME"
                fi

                eval alias_n='$HOST_'$i'_ALIAS_N'
                [ 0"$alias_n" -eq 0 ] || for j in `seq 1 $alias_n`
                do
                    eval alias='$HOST_'$i'_ALIAS_'$j
                    if [ "$alias" = "$hostname.$domain" ]
                    then
                       log_warn "Ignoring CNAME $alias because it is the same as the FQDN $hostname.$domain"
                    else
                        case $alias
                        in
                            *.*)
                                echo "cname=$alias,$hostname.$domain" >> /etc/dnsmasq.d/cname.conf
                                ;;
                            *)
                                echo "cname=$alias.$domain,$hostname.$domain" >> /etc/dnsmasq.d/cname.conf                            
                                ;;
                        esac
                    fi
                done

                eval ipaddr='$HOST_'$i'_IP4'
                if translate_ip_net $ipaddr HOST_${i}_IP4 
                then
                    echo "$res $hostname.$domain $hostname"
                    eval HOST_${i}_IP4="$res"
                fi
                if [ "$OPT_DNS" = "yes" -a "$DNS_SUPPORT_IPV6" = "yes" ]
                then
                    eval ipv6='$HOST_'$i'_IP6'
                    if [ "x$ipv6" != x -a "$ipv6" != "auto" ]
                    then
                        echo "$ipv6 $hostname.$domain $hostname"
                    else
                        echo "::ffff:$res $hostname.$domain $hostname"
                    fi
                fi
            done
        } >> $hostfile

        if [ -f /etc/dnsmasq.d/cname.conf ]
        then
            chown dns:dns /etc/dnsmasq.d/cname.conf
            chmod 600 /etc/dnsmasq.d/cname.conf
        fi

    fi
}

create_dns_hosts_extra ()
{
    if [ "$OPT_DNS" = "yes" ]
    then
        hostfile="/etc/hosts.d/hosts.extra"
    else
        hostfile="/etc/hosts"
    fi

    if [ 0$HOST_EXTRA_N -ne 0 ]
    then
        log_info "writing EXTRA_HOSTS to $hostfile ..."

        {
            cat <<EOF
#----------------------------------------------------------------------------
# $hostfile
# automatically created by rc001.hosts
#----------------------------------------------------------------------------
EOF
            [ 0$HOST_EXTRA_N -eq 0 ] || for i in `seq 1 $HOST_EXTRA_N`
            do
                eval hostname='$HOST_EXTRA_'$i'_NAME'
                eval ipv4='$HOST_EXTRA_'$i'_IP4'
                echo "$ipv4 $hostname"
                if [ "$OPT_DNS" = "yes" -a "$DNS_SUPPORT_IPV6" = "yes" ]
                then
                    eval ipv6='$HOST_EXTRA_'$i'_IP6'
                    if [ "x$ipv6" != x -a "$ipv6" != "auto" ]
                    then
                        echo "$ipv6 $hostname"
                    else
                        echo "::ffff:$ipv4 $hostname"
                    fi
                fi
            done
        } >> $hostfile
    fi
}
